#!/usr/bin/perl

use strict;
use v5.10;

use lib 'lib';
use Data::Dumper 'Dumper';
use WebService::Gitter;

my $git = WebService::Gitter->new(
    api_key => $ENV{GITTER_KEY}
);

my $user_id = $git->get_me->{id};
my $ujango_id = '5a5b8818d73408ce4f882f9d';
#my $current_user = $git->list_groups;

#say Dumper($git->rooms_under_group('5a561773d73408ce4f8765fe')->[0]{userCount});

#say Dumper($git->rooms);


#say Dumper $git->room_users('5880e3b6d73408ce4f44a1aa', q => 'faraco')->[0]{username};
#say Dumper $git->list_messages('5880e3b6d73408ce4f44a1aa', q => 'faraco')->[0]{id};
#say Dumper $git->single_message('5880e3b6d73408ce4f44a1aa', '591444f733e9ee771c96b8ca');
#say Dumper $git->send_message('5880e3b6d73408ce4f44a1aa', text => 'test only1.');
#say Dumper $git->update_message('5880e3b6d73408ce4f44a1aa', '5a5b75075a9ebe4f75865baa', text => 'test only2.');
#say Dumper $git->join_room($ujango_id, $user_id);
#say Dumper $git->remove_user_from_room('5a5b7b26d73408ce4f882e0c', '584ab515d73408ce4f3be368');
#say Dumper $git->delete_room('5a5b7b26d73408ce4f882e0c');



